@@ -1,17 +1,50 @@
-(*pp camlp4o -I `ocamlfind query sexplib` -I `ocamlfind query type-conv` -I `ocamlfind query bin_prot` pa_type_conv.cmo pa_sexp_conv.cmo pa_bin_prot.cmo *)
+(******************************************************************************
+ *                             Core                                           *
+ *                                                                            *
+ * Copyright (C) 2008- Jane Street Holding, LLC                               *
+ *    Contact: opensource@janestreet.com                                      *
+ *    WWW: http://www.janestreet.com/ocaml                                    *
+ *                                                                            *
+ *                                                                            *
+ * This library is free software; you can redistribute it and/or              *
+ * modify it under the terms of the GNU Lesser General Public                 *
+ * License as published by the Free Software Foundation; either               *
+ * version 2 of the License, or (at your option) any later version.           *
+ *                                                                            *
+ * This library is distributed in the hope that it will be useful,            *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
+ * Lesser General Public License for more details.                            *
+ *                                                                            *
+ * You should have received a copy of the GNU Lesser General Public           *
+ * License along with this library; if not, write to the Free Software        *
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
+ *                                                                            *
+ ******************************************************************************)
 
 (* Double-ended index-able queue *)
 
+(* the user's index is called the "conceptual" index (variable names are index or i),
+   while the corresponding index into the physical array is called the "physical" index
+   (variable names are pindex or p)
+   the physical length is "Array.length buf.data" (variable name plength) while the
+   conceptual length is buf.length (variable name length)
+   invariant: physical_index = conceptual_index mod physical_length
+   the "front" has smaller indices
+   physical array lengths are always powers of two
+   maybe I should insert dummy values into dropped indices to g.c. junk sooner? *)
+
+
+
 open Std_internal
 
-TYPE_CONV_PATH "Dequeue"
 
 type 'a t = { mutable data: 'a array;
-             mutable min_index: int; (* conceptual *)
-             mutable length: int;    (* conceptual: max_index = min_index + length - 1 *)
-             never_shrink: bool;
-             dummy: 'a;
-           }
+              mutable min_index: int; (* conceptual *)
+              mutable length: int;    (* conceptual: max_index = min_index + length - 1 *)
+              never_shrink: bool;
+              dummy: 'a;
+            }
  with sexp
 
 type 'a sexpable = 'a t
