@@ -1,3 +1,27 @@
+(******************************************************************************
+ *                             Core                                           *
+ *                                                                            *
+ * Copyright (C) 2008- Jane Street Holding, LLC                               *
+ *    Contact: opensource@janestreet.com                                      *
+ *    WWW: http://www.janestreet.com/ocaml                                    *
+ *                                                                            *
+ *                                                                            *
+ * This library is free software; you can redistribute it and/or              *
+ * modify it under the terms of the GNU Lesser General Public                 *
+ * License as published by the Free Software Foundation; either               *
+ * version 2 of the License, or (at your option) any later version.           *
+ *                                                                            *
+ * This library is distributed in the hope that it will be useful,            *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
+ * Lesser General Public License for more details.                            *
+ *                                                                            *
+ * You should have received a copy of the GNU Lesser General Public           *
+ * License along with this library; if not, write to the Free Software        *
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
+ *                                                                            *
+ ******************************************************************************)
+
 open Sexplib.Conv
 
 module Toggle = struct
@@ -107,7 +131,7 @@ module Timer = struct
       fail_alert: ('param -> 'alert);
       success_alert: ('param -> 'alert);
 
-      min_alert_interval: Time.Span.t;
+      min_alert_interval: Span.t;
       mutable prior_fail_alert_time : Time.t option;
     }
   let create ~assertion ~fail_alert ~success_alert ~min_alert_interval =
@@ -137,11 +161,10 @@ module Timer = struct
         ec.prior_fail_alert_time <- Some now;
         Some (ec.fail_alert param)
       in
-
       match ec.prior_fail_alert_time with
       | None -> fail_alert ()
       | Some t ->
-          if Time.abs_diff now t > ec.min_alert_interval then
+          if Span.(>) (Time.abs_diff now t) ec.min_alert_interval then
             (* enough time has passed since last failure, so alert *)
             fail_alert ()
           else
