@@ -1,18 +1,38 @@
-(** Module for easily generating unit tests.  Based on code posted by
-    padiolea\@irisa.fr to the caml mailing list. *)
+(******************************************************************************
+ *                             Core                                           *
+ *                                                                            *
+ * Copyright (C) 2008- Jane Street Holding, LLC                               *
+ *    Contact: opensource@janestreet.com                                      *
+ *    WWW: http://www.janestreet.com/ocaml                                    *
+ *                                                                            *
+ *                                                                            *
+ * This library is free software; you can redistribute it and/or              *
+ * modify it under the terms of the GNU Lesser General Public                 *
+ * License as published by the Free Software Foundation; either               *
+ * version 2 of the License, or (at your option) any later version.           *
+ *                                                                            *
+ * This library is distributed in the hope that it will be useful,            *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
+ * Lesser General Public License for more details.                            *
+ *                                                                            *
+ * You should have received a copy of the GNU Lesser General Public           *
+ * License along with this library; if not, write to the Free Software        *
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
+ *                                                                            *
+ ******************************************************************************)
 
 
 
 open Std_internal
 
+
+
 let rec foldn ~f ~init:acc i =
   if i = 0 then acc else foldn ~f ~init:(f acc i) (i-1)
 
-let sum_int = List.fold_left ~f:(+) ~init:0
+let sum_int = List.fold ~f:(+) ~init:0
 
-(*------------------------------------------------------------------------------*)
-(* generator *)
-(*------------------------------------------------------------------------------*)
 type 'a gen = unit -> 'a
 
 let pfg () = exp (Random.float 30. -. 15.)
@@ -20,7 +40,6 @@ let pfg () = exp (Random.float 30. -. 15.)
 let fg () =
   pfg () *. (if Random.bool () then 1. else -1.)
 
-(* natural number generator *)
 let nng () =
   let p = Random.float 1. in
   if p < 0.5 then Random.int 10
@@ -36,7 +55,7 @@ let nng () =
  *   min_int <= result <= max_int
  *)
 let uig =
-  let bound = Int64.add 1L (Int64.of_int max_int) in
+  let bound = Int64.(+) 1L (Int64.of_int max_int) in
   fun () ->
     let r = Int64.to_int_exn (Random.int64 bound) in
     if Random.bool () then r else -r - 1
@@ -57,16 +76,11 @@ let sg ?(char_gen = cg) ?(size_gen = nng) () =
   done;
   s
 
-(** Given a list of generators, returns generator that randomly uses one of the generators
-    from the list *)
 let oneof xs =
   List.nth_exn xs (Random.int (List.length xs))
 
-(** generator that always returns given value *)
 let always x () = x
 
-(** Given list of [(frequency,value)] pairs, returns value with probability proportional
-    to given frequency *)
 let frequency xs =
   let sums = sum_int (List.map ~f:fst xs) in
   let i = Random.int sums in
@@ -76,12 +90,8 @@ let frequency xs =
   in
   aux 0 xs
 
-(** like frequency, but returns generator *)
 let frequencyl l = frequency (List.map ~f:(fun (i,e) -> (i,always e)) l)
 
-(** [laws iter gen func] applies [func] repeatedly ([iter] times) on output of [gen], and
-    if [func] ever returns false, then the input that caused the failure is returned
-    optionally.  *)
 let rec laws iter gen func =
   if iter <= 0 then None
   else
@@ -91,31 +101,10 @@ let rec laws iter gen func =
       else laws (iter-1) gen func
     with _ -> Some input
 
-(** Like laws, but throws an exception instead of returning an option.  *)
 let laws_exn name iter gen func =
   match laws iter gen func with
     None -> ()
   | Some _ -> failwith (Printf.sprintf "law %s failed" name)
 
-let rec statistic_number = function
-  | []    -> []
-  | x::xs -> let (splitg, splitd) = List.partition ~f:(fun y -> y = x) xs in
-    (1 + List.length splitg, x) :: statistic_number splitd
-
-(* in percentage *)
-let statistic xs =
-  let stat_num = statistic_number xs in
-  let totals = sum_int (List.map ~f:fst stat_num) in
-  List.map ~f:(fun (i, v) -> ((i * 100) / totals), v) stat_num
-
-
-let laws2 iter func gen =
-  let res = foldn ~init:[] iter
-    ~f:(fun acc _ -> let n = gen () in (n, func n) :: acc)
-  in
-  let stat = statistic (List.map ~f:(fun (_, (_, v)) -> v) res) in
-  let res = List.filter ~f:(fun (_, (b, _)) -> not b) res in
-  if res = [] then (None, stat) else (Some (fst (List.hd_exn res)), stat)
-
 let repeat times test gen =
   for i = 1 to times do test (gen()) done
