@@ -1,3 +1,27 @@
+(******************************************************************************
+ *                             Core                                           *
+ *                                                                            *
+ * Copyright (C) 2008- Jane Street Holding, LLC                               *
+ *    Contact: opensource@janestreet.com                                      *
+ *    WWW: http://www.janestreet.com/ocaml                                    *
+ *                                                                            *
+ *                                                                            *
+ * This library is free software; you can redistribute it and/or              *
+ * modify it under the terms of the GNU Lesser General Public                 *
+ * License as published by the Free Software Foundation; either               *
+ * version 2 of the License, or (at your option) any later version.           *
+ *                                                                            *
+ * This library is distributed in the hope that it will be useful,            *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
+ * Lesser General Public License for more details.                            *
+ *                                                                            *
+ * You should have received a copy of the GNU Lesser General Public           *
+ * License along with this library; if not, write to the Free Software        *
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
+ *                                                                            *
+ ******************************************************************************)
+
 open OUnit;;
 open Core.Std
 
@@ -58,7 +82,8 @@ let test =
           let s_dup = "((a 1) (b 2) (a 3) (d 4))" in
           let s_dup = Sexp.of_string s_dup in
           assert_raises
-            (Sexplib.Conv.Of_sexp_error ("Map.t_of_sexp: duplicate key", (sexp_of_string "a")))
+            (Sexplib.Conv.Of_sexp_error (
+              Failure "Map.t_of_sexp: duplicate key", sexp_of_string "a"))
             (fun () -> Map.t_of_sexp string_of_sexp int_of_sexp s_dup)
         );
       "of_alist" >::
